/* omega copyright (c) 1987,1988,1989 by Laurence Raphael Brothers */
/* osave.c */

#include "oglob.h"
#ifndef MSDOS
#include <unistd.h>
#include <stdlib.h>
#endif

/*Various functions for doing game saves and restores */
/*The game remembers various player information, the city level,
the country level, and the last or current dungeon level */

/**************** SAVE FUNCTIONS */

/* Checks to see if save file already exists.
   Checks to see if save file can be opened for write.
   The player, the city level, and the current dungeon level are saved.
*/
    
int save_game(compress,savestr)
int compress;
char *savestr;
{
  FILE *fd;
  int slashpos;
#ifdef MSDOS
  int tmpdepth;
#endif
  int i,writeok=TRUE;
  char cmd[200];

  if (savestr[0] == '\0')
    writeok = FALSE;
#ifndef MSDOS
  else if (access(savestr, R_OK) == 0)
    if (access(savestr, W_OK) == 0)
    {
	mprint(" Overwrite old file?");
	writeok = (ynq() == 'y');
    }
    else
    {
	mprint(" File already exists.");
	writeok = FALSE;
    }
  else
  {
    for (slashpos = strlen(savestr); slashpos > 0 && savestr[slashpos] != '/';
	slashpos--)
	;
    if (slashpos > 0)
    {
	savestr[slashpos] = '\0';
	if (access(savestr, W_OK) == -1)
	{
	    mprint(" Unable to save to that directory.");
	    writeok = FALSE;
	}
	savestr[slashpos] = '/';
    }
  }
#endif
  change_to_user_perms();
  if (writeok) {
#ifndef MSDOS
    fd = fopen(savestr,"w");
#else
    fd=fopen(savestr,"wb");
#endif
    if (fd == NULL) {
      writeok = FALSE;
      mprint(" Error opening file.");
    }
  }
  if (! writeok)
  {
    morewait();
    print2("Save aborted.");
  }
  else {

    print1("Saving Game....");

    /* write the version number */
    i = VERSION;
    fwrite((char *)&i,sizeof(int),1,fd);
    /* write game id to save file */

    writeok &= save_player(fd);
    writeok &= save_country(fd);
#ifdef MSDOS
    tmpdepth = Level->depth;
    City = msdos_changelevel(Level,E_CITY,0);
#endif
    writeok &= save_level(fd,City);
    
    if ((Current_Environment != E_CITY) &&
	(Current_Environment != E_COUNTRYSIDE)) {
      i = TRUE;
      if (!fwrite((char *)&i,sizeof(int),1,fd))
	writeok = FALSE;
#ifdef MSDOS
      Level = msdos_changelevel(NULL,Current_Environment,tmpdepth);
#endif
      writeok &= save_level(fd,Level);
    }
    else {
      i = FALSE;
      if (!fwrite((char *)&i,sizeof(int),1,fd))
	writeok = FALSE;
    }
    fclose(fd);
    if (writeok)
	print1("Game Saved.");
    else
	print1("Something didn't work... save aborted.");

#ifdef COMPRESS_SAVE_FILES
    if (writeok && compress) {
      print2("Compressing Save File....");
      strcpy(cmd,"compress ");
      strcat(cmd,savestr);
      system(cmd);
#ifdef MSDOS
      if (strlen(savestr) < 13) {
#endif
	sprintf(cmd, "mv %s.Z %s", savestr, savestr);
	system(cmd);
#ifdef MSDOS
      }
#endif
    }
#endif
    morewait();
    clearmsg();
  }
  change_to_game_perms();
  return(writeok);
}




/* saves game on SIGHUP */
/* no longer tries to compress, which hangs */
void signalsave()
{
  int i;
  FILE *fd;

  change_to_user_perms();
#ifndef MSDOS
  fd=fopen("Omega.Sav","w");
#else
  fd=fopen("Omega.Sav","wb");
#endif
  if (fd != NULL) {
    /* write the version number */
    i = VERSION;
    fwrite((char *)&i,sizeof(int),1,fd);
    /* write game id to save file */

    save_player(fd);
    save_country(fd);
    save_level(fd,City);
    
    if ((Current_Environment != E_CITY) &&
	(Current_Environment != E_COUNTRYSIDE)) {
      i = TRUE;
      fwrite((char *)&i,sizeof(int),1,fd);
      save_level(fd,Level);
    }
    else {
      i = FALSE;
      fwrite((char *)&i,sizeof(int),1,fd);
    }
    fclose(fd);
#ifdef COMPRESS_SAVE_FILES
    print1("Signal - Saving uncompressed file 'Omega.Sav'.");
    print2("You should compress it yourself.");
    morewait();
#else
    print1("Signal - Saving file 'Omega.Sav'.");
    morewait();
#endif
  }
  endgraf();
  exit(0);
}


/* also saves some globals like Level->depth... */

int save_player(fd)
FILE *fd;
{
  int i;
  int ok = 1;

  /* Save random global state information */

  ok &= (fwrite((char *)&Player,sizeof(Player),1,fd) > 0);
#ifdef BSD
  ok &= (fprintf(fd,"%s\n",Password) != EOF);
  ok &= (fprintf(fd,"%s\n",Player.name) != EOF);
#else
  ok &= (fprintf(fd,"%s\n",Password) >= 0);
  ok &= (fprintf(fd,"%s\n",Player.name) >= 0);
#endif
  ok &= (fwrite((char *)CitySiteList,sizeof(CitySiteList),1,fd) > 0);
  ok &= (fwrite((char *)&GameStatus,sizeof(long),1,fd) > 0);
  ok &= (fwrite((char *)&Current_Environment,sizeof(int),1,fd) > 0);
  ok &= (fwrite((char *)&Last_Environment,sizeof(int),1,fd) > 0);
  ok &= (fwrite((char *)&Current_Dungeon,sizeof(int),1,fd) > 0);
  ok &= (fwrite((char *)&Villagenum,sizeof(int),1,fd) > 0);
  ok &= (fwrite((char *)&Verbosity,sizeof(char),1,fd) > 0);
#ifndef MSDOS
  ok &= (fwrite((char *)&Time,sizeof(int),1,fd) > 0);
#else
  ok &= (fwrite((char *)&Time,sizeof(long),1,fd) > 0);
#endif
  ok &= (fwrite((char *)&Tick,sizeof(int),1,fd) > 0);
  ok &= (fwrite((char *)&Searchnum,sizeof(int),1,fd) > 0);
  ok &= (fwrite((char *)&Phase,sizeof(int),1,fd) > 0);
  ok &= (fwrite((char *)&Date,sizeof(int),1,fd) > 0);
  ok &= (fwrite((char *)&Spellsleft,sizeof(int),1,fd) > 0);
  ok &= (fwrite((char *)&SymbolUseHour,sizeof(int),1,fd) > 0);
  ok &= (fwrite((char *)&ViewHour,sizeof(int),1,fd) > 0);
  ok &= (fwrite((char *)&HelmHour,sizeof(int),1,fd) > 0);
  ok &= (fwrite((char *)&Constriction,sizeof(int),1,fd) > 0);
  ok &= (fwrite((char *)&Blessing,sizeof(int),1,fd) > 0);
  ok &= (fwrite((char *)&LastDay,sizeof(int),1,fd) > 0);
  ok &= (fwrite((char *)&RitualHour,sizeof(int),1,fd) > 0);
  ok &= (fwrite((char *)&Lawstone,sizeof(int),1,fd) > 0);
  ok &= (fwrite((char *)&Chaostone,sizeof(int),1,fd) > 0);
  ok &= (fwrite((char *)&Mindstone,sizeof(int),1,fd) > 0);
  ok &= (fwrite((char *)&Arena_Opponent,sizeof(int),1,fd) > 0);
  ok &= (fwrite((char *)&Imprisonment,sizeof(int),1,fd) > 0);
#ifndef MSDOS
  ok &= (fwrite((char *)&Gymcredit,sizeof(int),1,fd) > 0);
#else
  ok &= (fwrite((char *)&Gymcredit,sizeof(long),1,fd) > 0);
#endif
#ifndef MSDOS
  ok &= (fwrite((char *)&Balance,sizeof(int),1,fd) > 0);
#else
  ok &= (fwrite((char *)&Balance,sizeof(long),1,fd) > 0);
#endif
  ok &= (fwrite((char *)&StarGemUse,sizeof(int),1,fd) > 0);
  ok &= (fwrite((char *)&HiMagicUse,sizeof(int),1,fd) > 0);
  ok &= (fwrite((char *)&HiMagic,sizeof(int),1,fd) > 0);
#ifndef MSDOS
  ok &= (fwrite((char *)&FixedPoints,sizeof(int),1,fd) > 0);
#else
  ok &= (fwrite((char *)&FixedPoints,sizeof(long),1,fd) > 0);
#endif
  ok &= (fwrite((char *)&LastCountryLocX,sizeof(int),1,fd) > 0);
  ok &= (fwrite((char *)&LastCountryLocY,sizeof(int),1,fd) > 0);
  ok &= (fwrite((char *)&LastTownLocX,sizeof(int),1,fd) > 0);
  ok &= (fwrite((char *)&LastTownLocY,sizeof(int),1,fd) > 0);
  ok &= (fwrite((char *)&Pawndate,sizeof(int),1,fd) > 0);

  ok &= (fwrite((char *)Spells,sizeof(Spells),1,fd) > 0);

  ok &= (fwrite((char *)&Command_Duration,sizeof(Command_Duration),1,fd) > 0);
  ok &= (fwrite((char *)&Precipitation,sizeof(Precipitation),1,fd) > 0);
  ok &= (fwrite((char *)&Lunarity,sizeof(Lunarity),1,fd) > 0);
  ok &= (fwrite((char *)&ZapHour,sizeof(ZapHour),1,fd) > 0);
  ok &= (fwrite((char *)&RitualRoom,sizeof(RitualRoom),1,fd) > 0);

  /* stuff which used to be statics */
  ok &= (fwrite((char *)&twiddle,sizeof(twiddle),1,fd) > 0);
  ok &= (fwrite((char *)&saved,sizeof(saved),1,fd) > 0);
  ok &= (fwrite((char *)&onewithchaos,sizeof(onewithchaos),1,fd) > 0);
  ok &= (fwrite((char *)&club_hinthour,sizeof(club_hinthour),1,fd) > 0);
  ok &= (fwrite((char *)&winnings,sizeof(winnings),1,fd) > 0);
  ok &= (fwrite((char *)&tavern_hinthour,sizeof(tavern_hinthour),1,fd) > 0);
  ok &= (fwrite((char *)scroll_ids,sizeof(scroll_ids),1,fd) > 0);
  ok &= (fwrite((char *)potion_ids,sizeof(potion_ids),1,fd) > 0);
  ok &= (fwrite((char *)stick_ids,sizeof(stick_ids),1,fd) > 0);
  ok &= (fwrite((char *)ring_ids,sizeof(ring_ids),1,fd) > 0);
  ok &= (fwrite((char *)cloak_ids,sizeof(cloak_ids),1,fd) > 0);
  ok &= (fwrite((char *)boot_ids,sizeof(boot_ids),1,fd) > 0);
  ok &= (fwrite((char *)deepest,sizeof(deepest),1,fd) > 0);

  /* Save player possessions */

  if (Player.possessions[O_READY_HAND] == Player.possessions[O_WEAPON_HAND])
    Player.possessions[O_READY_HAND] = NULL;
  for(i=0;i<MAXITEMS;i++) ok &= save_item(fd,Player.possessions[i]);
  for(i=0;i<MAXPACK;i++) ok &= save_item(fd,Player.pack[i]);
  for(i=0;i<PAWNITEMS;i++) ok &= save_item(fd,Pawnitems[i]);

  /* Save items in condo vault */
  ok &= save_itemlist(fd,Condoitems);

  /* Save player item knowledge */
  for (i=0;i<TOTALITEMS;i++)
    ok &= (fwrite((char *)&(Objects[i].known),sizeof(Objects[i].known),1,fd) > 0);
  return ok;
}


/* Save whatever is pointed to by level */
int save_level(fd,level)
FILE *fd;
plv level;
{
  int i,j;
  int ok = 1;

  ok &= (fwrite((char *)level,sizeof(levtype),1,fd) > 0);
#ifdef MSDOS
  for (i = 0; i < MAXWIDTH; i++)
      ok &= (fwrite((char *)level->site[i],MAXLENGTH * sizeof(loctype),1,fd) > 0);
#endif
  ok &= save_monsters(fd,level->mlist);
  for(i=0;i<MAXWIDTH;i++)
    for(j=0;j<MAXLENGTH;j++)
      ok &= save_itemlist(fd,level->site[i][j].things);
  return ok;
}


int save_monsters(fd,ml)
FILE *fd;
pml ml;
{
  pml tml;
  int nummonsters=0;
  int ok = 1;

  /* First count monsters */
  for(tml=ml;tml!=NULL;tml=tml->next) 
    if (tml->m->hp > 0) nummonsters++;

  ok &= (fwrite((char *)&nummonsters,sizeof(int),1,fd) > 0);

  /* Now save monsters */
  for(tml=ml;tml!=NULL;tml=tml->next) {
    if (tml->m->hp > 0) {
      ok &= (fwrite((char *)tml->m,sizeof(montype),1,fd) > 0);
#ifdef BSD
      ok &= (fprintf(fd,"%s\n",tml->m->monstring) != EOF);
#else
      ok &= (fprintf(fd,"%s\n",tml->m->monstring) >= 0);
#endif
      ok &= save_itemlist(fd,tml->m->possessions);
    }
  }
  return ok;
}


/* Save o unless it's null, then save a null item instead */
int save_item(fd,o)
FILE *fd;
pob o;
{
  objtype nullobject;
  int ok = 1;

  nullobject.id = -1;
  if (o == NULL) {
    ok &= (fwrite((char *)&nullobject,sizeof(objtype),1,fd) > 0);
#ifdef BSD
    ok &= (fprintf(fd,"Null Object. Report if you see this!\n") != EOF);
#else
    ok &= (fprintf(fd,"Null Object. Report if you see this!\n") >= 0);
#endif
  }
  else {
    ok &= (fwrite((char *)o,sizeof(objtype),1,fd) > 0);
#ifdef BSD
    ok &= (fprintf(fd,"%s\n",o->truename) != EOF);
#else
    ok &= (fprintf(fd,"%s\n",o->truename) >= 0);
#endif
  }
  return ok;
}

int save_itemlist(fd,ol)
FILE *fd;
pol ol;
{
  int numitems = 0;
  pol tol;
  int ok = 1;

  for(tol=ol;tol!=NULL;tol=tol->next) numitems++;
  ok &= (fwrite((char *)&numitems,sizeof(int),1,fd) > 0);
  for(tol=ol;tol!=NULL;tol=tol->next)
    ok &= save_item(fd,tol->thing);
  return ok;
}


int save_country(fd)
FILE *fd;
{
  return (fwrite((char *)Country,sizeof(Country),1,fd) > 0);
}




/* read player data, city level, dungeon level,
   check on validity of save file, etc.
   return TRUE if game restored, FALSE otherwise */

int restore_game(savestr)
char *savestr;
{
  int i,version; 
  char cmd[200];
  FILE *fd;

#ifndef MSDOS
  if (access(savestr, F_OK|R_OK|W_OK) == -1) /* access uses real uid */
  {
    print1("Unable to access save file: ");
    nprint1(savestr);
    morewait();
    return FALSE;
  }
#endif
  change_to_user_perms();
#ifdef COMPRESS_SAVE_FILES
  print1("Uncompressing Save File....");
#ifdef MSDOS
  if (strlen(savestr) < 13) {
#endif
    sprintf(cmd, "mv %s %s.Z", savestr, savestr);
    system(cmd);
#ifdef MSDOS
  }
#endif
  strcpy(cmd,"uncompress ");
  strcat(cmd,savestr);
  system(cmd);
  print2("Save file uncompressed.");
  morewait();
  if ((savestr[strlen(savestr)-1] == 'Z') &&
      (savestr[strlen(savestr)-2] == '.'))
    savestr[strlen(savestr)-2] = 0;
#endif
  
#ifndef MSDOS
  fd = fopen(savestr,"r");
#else
  fd = fopen(savestr,"rb");
#endif

  if (fd == NULL) {
    print1("Error restoring game -- aborted.");
    print2("File name was: ");
    nprint2(savestr);
    morewait();
    change_to_game_perms();
    return(FALSE);
  }
  else {
    print1("Restoring...");

    fread((char *)&version,sizeof(int),1,fd);

    if (VERSION != version) {
      print2(" Sorry, I can't restore an outdated save file!");
      morewait();
      change_to_game_perms();
      return(FALSE);
    }
    restore_player(fd);
    restore_country(fd);
    restore_level(fd); /* the city level */
    fread((char *)&i,sizeof(int),1,fd);
    if (i == TRUE)
#ifndef MSDOS
      restore_level(fd);
#else
    {
      msdos_changelevel(Level,0,-1);
      restore_level(fd);
    }
#endif
    print3("Restoration complete.");
    ScreenOffset = -1000;
    screencheck(Player.y);
    drawvision(Player.x,Player.y);
    setgamestatus(SKIP_MONSTERS);
    dataprint();
    showflags();
    fclose(fd);
    change_to_game_perms();
    return(TRUE);
  }
}



void restore_player(fd)
FILE *fd;
{
  int i;
  fread((char *)&Player,sizeof(Player),1,fd);
  filescanstring(fd,Password);
  filescanstring(fd,Player.name);
  fread((char *)CitySiteList,sizeof(CitySiteList),1,fd);
  fread((char *)&GameStatus,sizeof(long),1,fd);
  fread((char *)&Current_Environment,sizeof(int),1,fd);
  fread((char *)&Last_Environment,sizeof(int),1,fd);
  fread((char *)&Current_Dungeon,sizeof(int),1,fd);
  fread((char *)&Villagenum,sizeof(int),1,fd);
  switch(Current_Dungeon) {
  case E_ASTRAL: MaxDungeonLevels = ASTRALLEVELS; break;
  case E_SEWERS: MaxDungeonLevels = SEWERLEVELS; break;
  case E_CASTLE: MaxDungeonLevels = CASTLELEVELS; break;
  case E_CAVES: MaxDungeonLevels = CAVELEVELS; break;
  case E_VOLCANO: MaxDungeonLevels = VOLCANOLEVELS; break;
  }
  fread((char *)&Verbosity,sizeof(char),1,fd);
#ifndef MSDOS
  fread((char *)&Time,sizeof(int),1,fd);
#else
  fread((char *)&Time,sizeof(long),1,fd);
#endif
  fread((char *)&Tick,sizeof(int),1,fd);
  fread((char *)&Searchnum,sizeof(int),1,fd);
  fread((char *)&Phase,sizeof(int),1,fd);
  fread((char *)&Date,sizeof(int),1,fd);
  fread((char *)&Spellsleft,sizeof(int),1,fd);
  fread((char *)&SymbolUseHour,sizeof(int),1,fd);
  fread((char *)&ViewHour,sizeof(int),1,fd);
  fread((char *)&HelmHour,sizeof(int),1,fd);
  fread((char *)&Constriction,sizeof(int),1,fd);
  fread((char *)&Blessing,sizeof(int),1,fd);
  fread((char *)&LastDay,sizeof(int),1,fd);
  fread((char *)&RitualHour,sizeof(int),1,fd);
  fread((char *)&Lawstone,sizeof(int),1,fd);
  fread((char *)&Chaostone,sizeof(int),1,fd);
  fread((char *)&Mindstone,sizeof(int),1,fd);
  fread((char *)&Arena_Opponent,sizeof(int),1,fd);
  fread((char *)&Imprisonment,sizeof(int),1,fd);
#ifndef MSDOS
  fread((char *)&Gymcredit,sizeof(int),1,fd);
#else
  fread((char *)&Gymcredit,sizeof(long),1,fd);
#endif
#ifndef MSDOS
  fread((char *)&Balance,sizeof(int),1,fd);
#else
  fread((char *)&Balance,sizeof(long),1,fd);
#endif
  fread((char *)&StarGemUse,sizeof(int),1,fd);
  fread((char *)&HiMagicUse,sizeof(int),1,fd);
  fread((char *)&HiMagic,sizeof(int),1,fd);
#ifndef MSDOS
  fread((char *)&FixedPoints,sizeof(int),1,fd);
#else
  fread((char *)&FixedPoints,sizeof(long),1,fd);
#endif
  fread((char *)&LastCountryLocX,sizeof(int),1,fd);
  fread((char *)&LastCountryLocY,sizeof(int),1,fd);
  fread((char *)&LastTownLocX,sizeof(int),1,fd);
  fread((char *)&LastTownLocY,sizeof(int),1,fd);
  fread((char *)&Pawndate,sizeof(int),1,fd);

  fread((char *)Spells,sizeof(Spells),1,fd);

  fread((char *)&Command_Duration,sizeof(Command_Duration),1,fd);
  fread((char *)&Precipitation,sizeof(Precipitation),1,fd);
  fread((char *)&Lunarity,sizeof(Lunarity),1,fd);
  fread((char *)&ZapHour,sizeof(ZapHour),1,fd);
  fread((char *)&RitualRoom,sizeof(RitualRoom),1,fd);

  /* stuff which used to be statics */
  fread((char *)&twiddle,sizeof(twiddle),1,fd);
  fread((char *)&saved,sizeof(saved),1,fd);
  fread((char *)&onewithchaos,sizeof(onewithchaos),1,fd);
  fread((char *)&club_hinthour,sizeof(club_hinthour),1,fd);
  fread((char *)&winnings,sizeof(winnings),1,fd);
  fread((char *)&tavern_hinthour,sizeof(tavern_hinthour),1,fd);
  fread((char *)scroll_ids,sizeof(scroll_ids),1,fd);
  fread((char *)potion_ids,sizeof(potion_ids),1,fd);
  fread((char *)stick_ids,sizeof(stick_ids),1,fd);
  fread((char *)ring_ids,sizeof(ring_ids),1,fd);
  fread((char *)cloak_ids,sizeof(cloak_ids),1,fd);
  fread((char *)boot_ids,sizeof(boot_ids),1,fd);
  fread((char *)deepest,sizeof(deepest),1,fd);

  /* Set up the strings for the id's */
  inititem(0);

  for(i=0;i<MAXITEMS;i++) 
    Player.possessions[i] = restore_item(fd);

  if (!Player.possessions[O_READY_HAND] && Player.possessions[O_WEAPON_HAND] &&
    twohandedp(Player.possessions[O_WEAPON_HAND]->id))
    Player.possessions[O_READY_HAND] = Player.possessions[O_WEAPON_HAND];

  for(i=0;i<MAXPACK;i++) 
    Player.pack[i] = restore_item(fd);
  for(i=0;i<PAWNITEMS;i++) 
    Pawnitems[i] = restore_item(fd);
  Condoitems = restore_itemlist(fd);
  for (i=0;i<TOTALITEMS;i++) 
    fread((char *)&(Objects[i].known),sizeof(Objects[i].known),1,fd);
  calc_melee();
}



pob restore_item(fd)
FILE *fd;
{
  char tempstr[80];
  pob obj = ((pob) malloc(sizeof(objtype)));
  fread((char *)obj,sizeof(objtype),1,fd);
  filescanstring(fd,tempstr);
  obj->truename = salloc(tempstr);
  if (obj->id < 0) {
    free((char *) obj);
    return(NULL);
  }
  else {
    obj->objstr = Objects[obj->id].objstr;
    obj->cursestr = Objects[obj->id].cursestr;
    return(obj);
  }
}

pol restore_itemlist(fd)
FILE *fd;
{
  pol ol=NULL,cur=NULL,new=NULL;
  int i,numitems,firsttime=TRUE;
  fread((char *)&numitems,sizeof(int),1,fd);
  for(i=0;i<numitems;i++) {
    new = ((pol) malloc(sizeof(oltype)));
    new->thing = restore_item(fd);
    new->next = NULL;
    if (firsttime==TRUE) {
      ol = cur = new;
      firsttime = FALSE;
    }
    else {
      cur->next = new;
      cur = new;
    }
  }
  return(ol);
}








void restore_level(fd)
FILE *fd;
{
  int i,j;

#ifndef MSDOS
  Level = ((plv) malloc(sizeof(levtype)));
  
  fread((char *)Level,sizeof(levtype),1,fd);
#else
  plc *tmp = (plc *)malloc(MAXWIDTH * sizeof(plc));

  Level = &TheLevel;
  for (i = 0; i < MAXWIDTH; i++)
    tmp[i] = Level->site[i];
  fread((char *)Level,sizeof(levtype),1,fd);
  for (i = 0; i < MAXWIDTH; i++)
    fread((char *)(Level->site[i] = tmp[i]),MAXLENGTH * sizeof(loctype),1,fd);
  free(tmp);
#endif

  Level->next = NULL;

  restore_monsters(fd,Level);
  for(i=0;i<MAXWIDTH;i++)
    for(j=0;j<MAXLENGTH;j++)
      Level->site[i][j].things = restore_itemlist(fd);
  if (Level->environment == E_CITY) City = Level;
  else if (Level->environment == Current_Dungeon) Dungeon = Level;
}




void restore_monsters(fd,level)
FILE *fd;
plv level;
{
  pml ml=NULL;
  int i,nummonsters;
  char tempstr[80];

  level->mlist = NULL;

  fread((char *)&nummonsters,sizeof(int),1,fd);
  
  for(i=0;i<nummonsters;i++) {
    ml = ((pml) malloc(sizeof(mltype)));
    ml->m = ((pmt) malloc(sizeof(montype)));
    ml->next = NULL;
    fread((char *)ml->m,sizeof(montype),1,fd);
    filescanstring(fd,tempstr);
    ml->m->monstring = salloc(tempstr);
    ml->m->corpsestr = salloc(Monsters[ml->m->id].corpsestr);
    ml->m->meleestr = salloc(Monsters[ml->m->id].meleestr);
    level->site[ml->m->x][ml->m->y].creature = ml->m;
    ml->m->possessions = restore_itemlist(fd);
    ml->next = level->mlist;
    level->mlist = ml;
  }
}



void restore_country(fd)
FILE *fd;
{
  fread((char *)Country,sizeof(Country),1,fd);
}
